# Supplementary Information 1: R code that reads and cleans SAFRING data. 

# Title: Impacts of time, weather and raptors on a fynbos bird community body
# metric data from the SAFRING database.


## Here we read all the data supplied by Dane Paaijmans from SAFRING of a set of
## target birds ringed in the W Cape


# This file indicates the data cleaning and preparation steps. 
# Author: Alan Lee

### File modification log

# File initially created April 2016
# Updated for publication purposes June 2017
# Checked for repeatability Oct 2017
# Updated according to reviewer comments March 2019

### End File modification log

# 30 October. Here we introduce an example data set for demo purposes (created
# using file 9...R). This example dataset contains 50 000 of the original >200
# 000 records. Ringer ID and Ring number have been anonymised in the example data.

require(dplyr); require(ggplot2); library(broom)


wp <- read.csv("example_data.csv") 

str(wp) #214884 initial original dataframe size
# replace all \N with NA
wp[wp=="\\N"] <- NA
# Ensure all data is numeric
wp$Mass <- as.numeric(as.character(wp$Mass)); wp$Wing <- as.numeric(as.character(wp$Wing)); wp$Culmen <- as.numeric(as.character(wp$Culmen)); 
wp$Head <- as.numeric(as.character(wp$Head)); wp$Tarsus <- as.numeric(as.character(wp$Tarsus))
hist(wp$Mass)
View(group_by(wp, Spp)%>%filter(!is.na(Mass))%>%summarise(mean(Mass), max(Mass)))
# Fix common errors
wp$Mass[wp$Mass==0] <- NA
wp$Mass[wp$Mass>250] <- NA# No species weighs more than 250g in our candidate list
wp$Wing[wp$Wing==0] <- NA

wp <- filter(wp, !is.na(Mass)|!is.na(Wing)|!is.na(Tarsus))#reduce the file to exclude birds with no key metric data

summary(wp)
hist(wp$E)
hist(wp$S)

# clearly people have confused E and S in some cases. 
# We will need to select only those data W of 2100E, so we need to fix this
for (i in 1:nrow(wp)){
  if(wp$E[i]>3000){a <- wp$S[i];b <- wp$E[i];wp$E[i] <- a;wp$S[i] <- b}
}
rm(a, b, i)
hist(wp$E)
hist(wp$S)
### now select records from the area of the weather stations
wp <- filter(wp, E<2100, S>3000)
length(unique(wp$Spp)) #61 

# also select values with valid year/month
str(wp$Startdate)
which(wp$Startdate=="0000-00-00")
wp <- filter(wp, !Startdate=="0000-00-00")
wp <- filter(wp, !Startdate=="0001-01-01")
wp$Startdate <- as.character(wp$Startdate)
wp$Date <- as.Date(wp$Startdate,'%d/%m/%Y')
wp$Year <- as.numeric(format(wp$Date, '%Y'))
wp$Month <- as.numeric(format(wp$Date, '%m'))

table(wp$Year)
# Few early year records, filter for post 1981
wp <- filter(wp, Year>1981)
wp <- filter(wp, !Year%in%c(2016, 2102)) # remove some more errors, also 2016 as this was truncation year for weather data

str(wp) #

# remove species with low capture frequency i.e. current n > 200
selector <- wp%>%group_by(Spp)%>%summarise(n=n())%>%filter(n>200)
wp <- filter(wp, Spp%in%selector$Spp)

length(unique(wp$Spp))#26 species here, but 43 in full dataset
rm(selector)

# Here we deal with the very obvious measurement errors
# e.g. should be no mass >80g for csb
with(filter(wp, Spp==749), hist(Mass))


# We remove those measures outside the 99%quantile - this should be conservative
# as the original quantile definition will include the error outliers

# first create a reference df with spp#

checkmass <- wp%>%filter(!is.na(Mass))%>%group_by(Spp)%>%do(data.frame(lc=quantile(.$Mass, c(.005)), uq=quantile(.$Mass, c(.995))))
checkWing <- wp%>%filter(!is.na(Wing))%>%group_by(Spp)%>%do(data.frame(lcwing=quantile(.$Wing, c(.005)), uqwing=quantile(.$Wing, c(.995))))
checkTarsus <- wp%>%filter(!is.na(Tarsus))%>%group_by(Spp)%>%do(data.frame(lctarsus=quantile(.$Tarsus, c(.005)), uqtarsus=quantile(.$Tarsus, c(.995))))

checkdf <- left_join(checkmass, checkWing)
checkdf <- left_join(checkdf, checkTarsus)
rm(checkmass, checkWing, checkTarsus)
wp$masscheck <- NA
wp$wingcheck <- NA

# Warning: this loop code below is very slow! 

for(i in 1:nrow(wp)){
  #get matching row in checkdf
  check <- filter(checkdf, Spp==wp$Spp[i])
  
  ifelse(wp$Mass[i]>check$lc & wp$Mass[i]<check$uq, wp$masscheck[i] <- "OK", wp$masscheck[i] <- "Out")
  ifelse(wp$Wing[i]>check$lcwing & wp$Wing[i]<check$uqwing, wp$wingcheck[i] <- "OK", wp$wingcheck[i] <- "Out")
  ifelse(wp$Tarsus[i]>check$lctarsus & wp$Tarsus[i]<check$uqtarsus, wp$Tarsus[i] <- wp$Tarsus[i], wp$Tarsus[i] <- NA)
  
}
rm(check, checkdf, i)


nrow(wp%>%filter(masscheck=="Out"&wingcheck=="Out"))#double error - let us remove these. Will include chicks etc.
wp <- filter(wp, !(masscheck=="Out"&wingcheck=="Out"))

nrow(wp%>%filter(masscheck=="OK"&wingcheck=="OK"))# good data n original = 84451


# In the SAFRING database, 1 = male, 2 = female, 3 = possible male, 4 = possible female, 0 = unknown sex
table(wp$Sex)
wp$Sex[wp$Sex>4] <- 0 # more errors, probably from mixing sex with Age. Here we recode some to 0 (SAFRING code for unknown)


# Add species names
adu_names <- read.csv("adu_names.csv", stringsAsFactors = F)
wp$common_name <- NA
for(i in 1:nrow(wp)){
  wp$common_name[i] <-   adu_names[which(wp$Spp[i]==adu_names$number), "common_name"]
}

unique(wp$common_name)  
# we learn later that species 1172 (Cape white-eye) and 4139 (Spotted/Karoo Prinia)
# were split during the study period, so data is only partial: drop these:
wp <- filter(wp, !Spp%in%c(1172, 4139))

# what data are we left with?
wp%>%group_by(Spp)%>%filter(!is.na(Mass))%>%summarise(nMass=n())
wp%>%group_by(Spp)%>%filter(!is.na(Tarsus))%>%summarise(nTarsus=n())



# Some exploration on the final data

Total=wp%>%summarise(Total=n()) 
Mass=wp%>%filter(!is.na(Mass))%>%summarise(Mass=n()) 
Wing=wp%>%filter(!is.na(Wing))%>%summarise(Wing=n()) 
Tarsus=wp%>%filter(!is.na(Tarsus))%>%summarise(Tarsus=n()) 
temp <- data.frame(Total, Mass, Wing, Tarsus)
temp <- unlist(temp)

barplot(temp, ylab=c("n")) #


# Effort by year
barplot(table(wp$Year))

rm(Total,Mass, Tarsus, Wing, temp, i)

# Illustrate the influence of ringer on measurements. This needs to be treated in as a random effect in models. 

boxplot(Mass~Primary_Ringer, data=filter(wp, Spp==799&masscheck=="OK"), main="Cape Weaver mass - Illustrating the influence of Ringer", xlab="Ringer unique ID", ylab="mass (g)")
boxplot(Mass~Primary_Ringer, data=filter(wp, Spp==749&masscheck=="OK"), main="Cape Sugarbird mass", xlab="Ringer unique ID", ylab="g")


## Bodycondition (BCI)

# This is the Scaled Mass Index (Peig and Green 2009)

# we need bSMA and L0. 
# L0 can be arithmetic mean of length (here we use Wing)

L0 <- filter(wp, Age==4&as.character(CODE)=="1"&!is.na(Wing))%>%group_by(common_name)%>%summarise( Lo = mean(Wing))
# initiate a dataframe
targetspecies <- L0$common_name
temp <- data.frame(targetspecies)

# for each species, calculate the bSMA
library(lmerTest)


for (i in 1:length(targetspecies)){
  temp$bSMA[i] <- 
    vcov(lmer(log(abs(Mass)) ~ log(Wing)+(1|Primary_Ringer), data = filter(wp, Age==4&common_name== targetspecies[i])))[2,1]/ #bOLS
    with(data = filter(wp, !is.na(Wing)&!is.na(Mass)&Age==4& common_name== targetspecies[i]), cor(log(abs(Mass)),log(Wing)))
  
  temp$common_name[i] <- targetspecies[i]
}
temp <- left_join(temp, L0)

wp$BCI <- NA 

# initiate loop to replace BC
for(i in 1:nrow(temp)){
  wp$BCI[wp$common_name ==temp$common_name[i]] <- wp$Mass [wp$common_name ==temp$common_name[i]]* ((temp$Lo[i]/wp$Wing[wp$common_name ==temp$common_name[i]]))^temp$bSMA[i]
}
rm(i, spp, L0)

# example distribution
with(filter(wp, common_name=="Cape Sugarbird"), hist(BCI))

# Here we explore if Wing length is a function of time (as year)

interAnnualSummaryByRinger <- wp%>%filter(Age==4&masscheck=="OK")%>%group_by(Spp, Primary_Ringer, Year)%>%summarise(mMass=mean(na.omit(Mass)),mWing=mean(na.omit(Wing))) 

temp <- interAnnualSummaryByRinger %>% group_by(Spp) %>% do(fit=summary(lmer(mWing ~ Year+(1|Primary_Ringer), data=.))) # same results 

targetspecies <- unique(interAnnualSummaryByRinger$Spp)
t <- temp$fit[[1]]
spp <- targetspecies[1]
DF <- data.frame(t$coefficients)
DF$species <- spp

for(i in 2:length(targetspecies)){
  t <- temp$fit[[i]]
  spp <- targetspecies[i]
  df <- data.frame(t$coefficients)
  df$species <- spp
  DF <- rbind(DF, df)
}
rm(df, t, spp)

DF$posneg <-  ifelse(DF$Estimate>0, "Positive", "Negative")
DF$plessthan05 <- ifelse(DF[,5]<0.00125, "Significant", "NS")
filter(DF, plessthan05!="NS") # No species have significant wing changes explained by Year

# Reviewer wants table of effort by species by site per year (although this region is 1 site in our treatment)

select(wp, common_name, Location_code, Year) %>% group_by(common_name, Location_code, Year) %>%tally

# wp was then joined to weather data for further analysis. The weather data was
# acquired from South African Weather Service, and needs to be requested
# directly as cannot provide it as we signed a use agreement. However, we
# generate some random pseudo data based on the lm regression of mean T by year
# to illustrate further modelling selection criteria.

wp2 <- wp
wp2$T <- rnorm(nrow(wp2), mean = 0.0035, sd = 0.0005) * wp2$Year + -5.8 +rnorm(nrow(wp2), mean=10, sd=5)
hist(wp2$T)

# random rainfall, rpois used to avoid n
wp2$rain <- rnorm(nrow(wp2), mean=2, sd=2)
wp2$rain <- ifelse(wp2$rain<0, 0, wp2$rain)
hist(wp2$rain)

# example of interannual analysis (simplified)
summary2 <- wp2%>%filter(CODE==1)%>%group_by(Spp, Year)%>%summarise(mBCI=mean(BCI, na.rm=T), mMass=mean(Mass, na.rm=T),mWing=mean(Wing, na.rm = T),  temperature=mean(T), rain=mean(rain) )
head(summary2)
#mean Wing as a function of time annual basis
interAnnualSummaryByRinger <- wp2%>%filter(Age==4&wingcheck=="OK"&CODE==1)%>%group_by(Spp, Primary_Ringer,  Year)%>%summarise(mWing=mean(Wing, na.rm = T), temperature=mean(T) ) 
temp <- interAnnualSummaryByRinger %>% group_by(Spp) %>% do(fit=summary(lmer(mWing ~ scale(Year)*scale(temperature)+(1|Primary_Ringer), data=.)))
temp[[2]]
